@@ -1,5 +1,6 @@
+"""pylandstats init."""
 from .landscape import *
 from .spatiotemporal import *
 from .zonal import *
 
-__version__ = '2.1.1'
+__version__ = "2.4.2"
