@@ -1,43 +1,41 @@
+"""Multi-landscape analysis."""
 import abc
 import functools
 
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
-import six
 
 from . import landscape as pls_landscape
 from . import settings
 
 _compute_class_metrics_df_doc = """
-Computes the data frame of class-level metrics, which is {index_descr}.
+Compute the data frame of class-level metrics, which is {index_descr}.
 
 Parameters
 ----------
 metrics : list-like, optional
-    A list-like of strings with the names of the metrics that should be
-    computed in the context of this analysis case
+    A list-like of strings with the names of the metrics that should be computed in the
+    context of this analysis case.
 classes : list-like, optional
-    A list-like of ints or strings with the class values that should be
-    considered in the context of this analysis case
+    A list-like of ints or strings with the class values that should be considered in
+    the context of this analysis case.
 metrics_kws : dict, optional
-    Dictionary mapping the keyword arguments (values) that should be passed to
-    each metric method (key), e.g., to exclude the boundary from the
-    computation of `total_edge`, metric_kws should map the string 'total_edge'
-    (method name) to {{'count_boundary': False}}. The default empty dictionary
-    will compute each metric according to FRAGSTATS defaults.
+    Dictionary mapping the keyword arguments (values) that should be passed to each
+    metric method (key), e.g., to exclude the boundary from the computation of
+    `total_edge`, metric_kws should map the string 'total_edge' (method name) to
+    {{'count_boundary': False}}. The default empty dictionary will compute each metric
+    according to FRAGSTATS defaults.
 fillna : bool, optional
-    Whether `NaN` values representing landscapes with no occurrences of
-    patches of the provided class should be replaced by zero when appropriate,
-    e.g., area and edge metrics (no ocurrences mean zero area/edge). If the
-    provided value is `None` (default), the value will be taken from
-    `settings.CLASS_METRICS_DF_FILLNA`.
+    Whether `NaN` values representing landscapes with no occurrences of patches of the
+    provided class should be replaced by zero when appropriate, e.g., area and edge
+    metrics (no ocurrences mean zero area/edge). If the provided value is `None`
+    (default), the value will be taken from `settings.CLASS_METRICS_DF_FILLNA`.
 
 Returns
 -------
 df : pandas.DataFrame
-    Dataframe with the values computed for each {index_return} and metric
-    (columns)
+    Dataframe with the values computed for each {index_return} and metric (columns).
 """
 
 _compute_landscape_metrics_df_doc = """
@@ -46,49 +44,60 @@ Computes the data frame of landscape-level metrics, which is {index_descr}.
 Parameters
 ----------
 metrics : list-like, optional
-    A list-like of strings with the names of the metrics that should be
-    computed. If None, all the implemented landscape-level metrics will be
-    computed.
+    A list-like of strings with the names of the metrics that should be computed. If
+    `None`, all the implemented landscape-level metrics will be computed.
 metrics_kws : dict, optional
-    Dictionary mapping the keyword arguments (values) that should be passed to
-    each metric method (key), e.g., to exclude the boundary from the
-    computation of `total_edge`, metric_kws should map the string 'total_edge'
-    (method name) to {{'count_boundary': False}}. The default empty dictionary
-    will compute each metric according to FRAGSTATS defaults.
+    Dictionary mapping the keyword arguments (values) that should be passed to each
+    metric method (key), e.g., to exclude the boundary from the computation of
+    `total_edge`, metric_kws should map the string 'total_edge' (method name) to
+    {{'count_boundary': False}}. The default empty dictionary will compute each metric
+    according to FRAGSTATS defaults.
 
 Returns
 -------
 df : pandas.DataFrame
-    Dataframe with the values computed at the landscape level for each
-    {index_return} and metric (columns)
+    Dataframe with the values computed at the landscape level for each {index_return}
+    and metric (columns).
 """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class MultiLandscape:
+class MultiLandscape(abc.ABC):
+    """Multi-landscape base abstract class."""
+
     @abc.abstractmethod
-    def __init__(self, landscapes, attribute_name, attribute_values):
+    def __init__(self, landscapes, attribute_name, attribute_values, **landscape_kws):
         """
+        Initialize the multi-landscape instance.
+
         Parameters
         ----------
         landscapes : list-like
-            A list-like of `Landscape` objects or of strings/file objects/
-            pathlib.Path objects so that each is passed as the `landscape`
-            argument of `Landscape.__init__`
+            A list-like of `Landscape` instances or of strings/file-like/pathlib.Path
+            objects so that each is passed as the `landscape` argument of
+            `Landscape.__init__`.
         attribute_name : str
-            Name of the attribute that will distinguish each landscape
+            Name of the attribute that will distinguish each landscape.
         attribute_values : list-like
-            Values of the attribute that are characteristic to each landscape
+            Values of the attribute that are characteristic to each landscape.
+        landscape_kws : dict, optional
+            Keyword arguments to be passed to the instantiation of
+            `pylandstats.Landscape` for each element of `landscapes`. Ignored if the
+            elements of `landscapes` are already instances of `pylandstats.Landcape`.
         """
         if isinstance(landscapes[0], pls_landscape.Landscape):
             self.landscapes = landscapes
         else:
-            self.landscapes = list(map(pls_landscape.Landscape, landscapes))
+            self.landscapes = [
+                pls_landscape.Landscape(landscape, **landscape_kws)
+                for landscape in landscapes
+            ]
 
         if len(self.landscapes) != len(attribute_values):
             raise ValueError(
                 "The lengths of `landscapes` and `{}` must coincide".format(
-                    attribute_name))
+                    attribute_name
+                )
+            )
 
         # set a `attribute_name` attribute with the value `attribute_values`,
         # so that children classes can access it (e.g., for
@@ -99,12 +108,13 @@ class MultiLandscape:
         # also set a `attribute_name` attribute so that the methods of this
         # class know how to access such attribute, i.e., as in
         # `getattr(self, self.attribute_name)`
-        setattr(self, 'attribute_name', attribute_name)
+        setattr(self, "attribute_name", attribute_name)
 
         # get the all classes present in the provided landscapes
         self.present_classes = functools.reduce(
             np.union1d,
-            tuple(landscape.classes for landscape in self.landscapes))
+            tuple(landscape.classes for landscape in self.landscapes),
+        )
 
     # fillna for metrics in class metrics dataframes. Since some classes might
     # not apprear in some of the landscapes (e.g., zones or temporal snapshots
@@ -116,21 +126,27 @@ class MultiLandscape:
     METRIC_FILLNA_DICT = {
         metric: 0
         for metric in [
-            patch_metric + '_' + suffix
-            for patch_metric in ['area', 'perimeter']
-            for suffix in ['mn', 'am', 'md', 'ra', 'sd']
-        ] + [
-            'total_area', 'proportion_of_landscape', 'number_of_patches',
-            'patch_density', 'largest_patch_index', 'total_edge',
-            'edge_density'
+            patch_metric + "_" + suffix
+            for patch_metric in ["area", "perimeter"]
+            for suffix in ["mn", "am", "md", "ra", "sd"]
+        ]
+        + [
+            "total_area",
+            "proportion_of_landscape",
+            "number_of_patches",
+            "patch_density",
+            "largest_patch_index",
+            "total_edge",
+            "edge_density",
         ]
     }
 
-    def __len__(self):
+    def __len__(self):  # noqa: D105
         return len(self.landscapes)
 
-    def compute_class_metrics_df(self, metrics=None, classes=None,
-                                 metrics_kws=None, fillna=None):
+    def compute_class_metrics_df(  # noqa: D102
+        self, metrics=None, classes=None, metrics_kws=None, fillna=None
+    ):
         attribute_values = getattr(self, self.attribute_name)
 
         # get the columns to init the data frame
@@ -162,37 +178,41 @@ class MultiLandscape:
         # TODO: one-level index if only one class?
         class_metrics_df = pd.DataFrame(
             index=pd.MultiIndex.from_product([classes, attribute_values]),
-            columns=columns)
+            columns=columns,
+        )
 
-        class_metrics_df.index.names = 'class_val', self.attribute_name
-        class_metrics_df.columns.name = 'metric'
+        class_metrics_df.index.names = "class_val", self.attribute_name
+        class_metrics_df.columns.name = "metric"
 
-        for attribute_value, landscape in zip(attribute_values,
-                                              self.landscapes):
+        for attribute_value, landscape in zip(attribute_values, self.landscapes):
             # get the class metrics DataFrame for the landscape that
             # corresponds to this attribute value
-            df = landscape.compute_class_metrics_df(metrics=metrics,
-                                                    metrics_kws=metrics_kws)
+            df = landscape.compute_class_metrics_df(
+                metrics=metrics, metrics_kws=metrics_kws
+            )
             # filter so we only check the classes considered in this
             # `MultiLandscape` instance
             df = df.loc[df.index.intersection(classes)]
             # put every row of the filtered DataFrame of this particular
             # attribute value
             for class_val, row in df.iterrows():
-                class_metrics_df.loc[(class_val,
-                                      attribute_value), columns] = row
+                class_metrics_df.loc[(class_val, attribute_value), columns] = row
 
         class_metrics_df = class_metrics_df.apply(pd.to_numeric)
         if fillna:
             class_metrics_df = class_metrics_df.fillna(
-                MultiLandscape.METRIC_FILLNA_DICT)
+                MultiLandscape.METRIC_FILLNA_DICT
+            )
         return class_metrics_df
 
     compute_class_metrics_df.__doc__ = _compute_class_metrics_df_doc.format(
-        index_descr='multi-indexed by the class and attribute value',
-        index_return='class, attribute value (multi-index)')
+        index_descr="multi-indexed by the class and attribute value",
+        index_return="class, attribute value (multi-index)",
+    )
 
-    def compute_landscape_metrics_df(self, metrics=None, metrics_kws=None):
+    def compute_landscape_metrics_df(  # noqa: D102
+        self, metrics=None, metrics_kws=None
+    ):
         attribute_values = getattr(self, self.attribute_name)
 
         # get the columns to init the data frame
@@ -211,60 +231,71 @@ class MultiLandscape:
             index = attribute_values
         landscape_metrics_df = pd.DataFrame(index=index, columns=columns)
         landscape_metrics_df.index.name = self.attribute_name
-        landscape_metrics_df.columns.name = 'metric'
+        landscape_metrics_df.columns.name = "metric"
 
-        for attribute_value, landscape in zip(attribute_values,
-                                              self.landscapes):
-            landscape_metrics_df.loc[attribute_value, columns] = \
-                landscape.compute_landscape_metrics_df(
-                    metrics,
-                    metrics_kws=metrics_kws).iloc[0]
+        for attribute_value, landscape in zip(attribute_values, self.landscapes):
+            landscape_metrics_df.loc[
+                attribute_value, columns
+            ] = landscape.compute_landscape_metrics_df(
+                metrics, metrics_kws=metrics_kws
+            ).iloc[
+                0
+            ]
 
         return landscape_metrics_df.apply(pd.to_numeric)
 
-    compute_landscape_metrics_df.__doc__ = \
-        _compute_landscape_metrics_df_doc.format(
-            index_descr='indexed by the attribute value',
-            index_return='attribute value (index)')
+    compute_landscape_metrics_df.__doc__ = _compute_landscape_metrics_df_doc.format(
+        index_descr="indexed by the attribute value",
+        index_return="attribute value (index)",
+    )
 
-    def plot_metric(self, metric, class_val=None, ax=None, metric_legend=True,
-                    metric_label=None, fmt='--o', plot_kws=None,
-                    subplots_kws=None, metric_kws=None):
+    def plot_metric(
+        self,
+        metric,
+        class_val=None,
+        ax=None,
+        metric_legend=True,
+        metric_label=None,
+        fmt="--o",
+        plot_kws=None,
+        subplots_kws=None,
+        metric_kws=None,
+    ):
         """
+        Plot the metric.
+
         Parameters
         ----------
         metric : str
-            A string indicating the name of the metric to plot
+            A string indicating the name of the metric to plot.
         class_val : int, optional
-            If provided, the metric will be plotted at the level of the
-            corresponding class, otherwise it will be plotted at the landscape
-            level
+            If provided, the metric will be plotted at the level of the corresponding
+            class, otherwise it will be plotted at the landscape level.
         ax : axis object, optional
-            Plot in given axis; if None creates a new figure
+            Plot in given axis; if None creates a new figure.
         metric_legend : bool, default True
-            Whether the metric label should be displayed within the plot (as
-            label of the y-axis)
+            Whether the metric label should be displayed within the plot (as label of
+            the y-axis).
         metric_label : str, optional
-            Label of the y-axis to be displayed if `metric_legend` is `True`.
-            If the provided value is `None`, the label will be taken from the
-            `settings` module
+            Label of the y-axis to be displayed if `metric_legend` is `True`. If the
+            provided value is `None`, the label will be taken from the `settings`
+            module.
         fmt : str, default '--o'
-            A format string for `matplotlib.pyplot.plot`
+            A format string for `matplotlib.pyplot.plot`.
         plot_kws : dict, default None
-            Keyword arguments to be passed to `matplotlib.pyplot.plot`
+            Keyword arguments to be passed to `matplotlib.pyplot.plot`.
         subplots_kws : dict, default None
-            Keyword arguments to be passed to `matplotlib.pyplot.plot.subplots`
-            only if no axis is given (through the `ax` argument)
+            Keyword arguments to be passed to `matplotlib.pyplot.plot.subplots` only if
+            no axis is given (through the `ax` argument).
         metric_kws : dict, default None
-            Keyword arguments to be passed to the method that computes the
-            metric (specified in the `metric` argument) for each landscape
+            Keyword arguments to be passed to the method that computes the metric
+            (specified in the `metric` argument) for each landscape.
 
         Returns
         -------
         ax : matplotlib.axes.Axes
-            Returns the `Axes` object with the plot drawn onto it
+            Returns the `Axes` object with the plot drawn onto it.
         """
-
         # TODO: metric_legend parameter acepting a set of str values
         # indicating, e.g., whether the metric label should appear as legend
         # or as yaxis label
@@ -280,28 +311,37 @@ class MultiLandscape:
                     for landscape in self.landscapes
                 ]
             except AttributeError:
-                raise ValueError("{metric} is not among {metrics}".format(
-                    metric=metric,
-                    metrics=pls_landscape.Landscape.CLASS_METRICS))
+                raise ValueError(
+                    "{metric} is not among {metrics}".format(
+                        metric=metric,
+                        metrics=pls_landscape.Landscape.CLASS_METRICS,
+                    )
+                )
             except TypeError:
                 raise ValueError(
-                    "{metric} cannot be computed at the landscape level".
-                    format(metric=metric))
+                    "{metric} cannot be computed at the landscape level".format(
+                        metric=metric
+                    )
+                )
         else:
             try:
                 metric_values = [
-                    getattr(landscape, metric)(class_val=class_val,
-                                               **metric_kws)
+                    getattr(landscape, metric)(class_val=class_val, **metric_kws)
                     for landscape in self.landscapes
                 ]
             except AttributeError:
-                raise ValueError("{metric} is not among {metrics}".format(
-                    metric=metric,
-                    metrics=pls_landscape.Landscape.LANDSCAPE_METRICS))
+                raise ValueError(
+                    "{metric} is not among {metrics}".format(
+                        metric=metric,
+                        metrics=pls_landscape.Landscape.LANDSCAPE_METRICS,
+                    )
+                )
             except TypeError:
                 raise ValueError(
                     "{metric} cannot be computed at the class level".format(
-                        metric=metric))
+                        metric=metric
+                    )
+                )
 
         if ax is None:
             if subplots_kws is None:
@@ -327,32 +367,37 @@ class MultiLandscape:
 
         return ax
 
-    def plot_landscapes(self, cmap=None, legend=True, subplots_kws=None,
-                        show_kws=None, subplots_adjust_kws=None):
+    def plot_landscapes(
+        self,
+        cmap=None,
+        legend=True,
+        subplots_kws=None,
+        show_kws=None,
+        subplots_adjust_kws=None,
+    ):
         """
-        Plots each landscape snapshot in a dedicated matplotlib axis by means
-        of the `Landscape.plot_landscape` method of each instance
+        Plot each landscape snapshot in a dedicated matplotlib axis.
+
+        Uses the `Landscape.plot_landscape` method of each instance.
 
         Parameters
-        -------
+        ----------
         cmap : str or `~matplotlib.colors.Colormap`, optional
-            A Colormap instance
+            A Colormap instance.
         legend : bool, optional
-            If ``True``, display the legend of the land use/cover color codes
-        subplots_kws: dict, default None
-            Keyword arguments to be passed to `matplotlib.pyplot.subplots`
-        show_kws : dict, default Nonte
-            Keyword arguments to be passed to `rasterio.plot.show`
-        subplots_adjust_kws: dict, default None
-            Keyword arguments to be passed to
-            `matplotlib.pyplot.subplots_adjust`
+            If ``True``, display the legend of the land use/cover color codes.
+        subplots_kws : dict, default None
+            Keyword arguments to be passed to `matplotlib.pyplot.subplots`.
+        show_kws : dict, default None
+            Keyword arguments to be passed to `rasterio.plot.show`.
+        subplots_adjust_kws : dict, default None
+            Keyword arguments to be passed to `matplotlib.pyplot.subplots_adjust`.
 
         Returns
         -------
         fig : matplotlib.figure.Figure
-            The figure with its corresponding plots drawn into its axes
+            The figure with its corresponding plots drawn into its axes.
         """
-
         attribute_values = getattr(self, self.attribute_name)
 
         # avoid alias/refrence issues
@@ -360,20 +405,22 @@ class MultiLandscape:
             _subplots_kws = {}
         else:
             _subplots_kws = subplots_kws.copy()
-        figsize = _subplots_kws.pop('figsize', None)
+        figsize = _subplots_kws.pop("figsize", None)
         if figsize is None:
-            figwidth, figheight = plt.rcParams['figure.figsize']
+            figwidth, figheight = plt.rcParams["figure.figsize"]
             figsize = (figwidth * len(attribute_values), figheight)
 
-        fig, axes = plt.subplots(1, len(attribute_values), figsize=figsize,
-                                 **_subplots_kws)
-
+        fig, axes = plt.subplots(
+            1, len(attribute_values), figsize=figsize, **_subplots_kws
+        )
+        if len(axes) == 1:  # len(attribute_values) == 1
+            axes = [axes]
         if show_kws is None:
             show_kws = {}
-        for attribute_value, landscape, ax in zip(attribute_values,
-                                                  self.landscapes, axes):
-            ax = landscape.plot_landscape(cmap=cmap, ax=ax, legend=legend,
-                                          **show_kws)
+        for attribute_value, landscape, ax in zip(
+            attribute_values, self.landscapes, axes
+        ):
+            ax = landscape.plot_landscape(cmap=cmap, ax=ax, legend=legend, **show_kws)
             ax.set_title(attribute_value)
 
         # adjust spacing between axes
